
% Skrypt tworzy strunkurę sieci konwolucyjnej i przeprowadza proces 
% uczenia dla bazy MNIST (80% próbek w zbiorze uczącym).
% Nauczona sieć jest sprawdzana na zbiorze testowym 
% (20% próbek z bazy MNIST)

clear; close all; clc


% Pobranie danych (cyfry z bazy MNIST)
digitDatasetPath = fullfile(matlabroot, 'toolbox','nnet', 'nndemos',...
                           'nndatasets','DigitDataset');
imds = imageDatastore(digitDatasetPath, 'IncludeSubfolders',true, ...
                     'LabelSource','foldernames');


% Definicja struktury CNN
layers = [
    imageInputLayer([28 28 1])
    convolution2dLayer([3 3], 3, "Padding","same")
    reluLayer
    maxPooling2dLayer(2, "Stride", 2)
    flattenLayer
    fullyConnectedLayer(10)
    softmaxLayer
    classificationLayer];
% graficzna prezentacja struktury CNN
figure; plot(layerGraph(layers));


% Przygotowanie danych i uczenie
[imdsTrain,imdsTest] = splitEachLabel(imds, 0.8, 'randomize');

options = trainingOptions("adam", Plots="training-progress");
[net, info] = trainNetwork(imdsTrain, layers, options);
accTrain = info.TrainingAccuracy(end)


% Sprawdzenie zbioru testowego
YPred = classify(net,imdsTest);
correct = (YPred == imdsTest.Labels);
accTest = sum(correct)/numel(imdsTest.Labels)
figure; confusionchart(YPred, imdsTest.Labels)
